//***********************************************************************************
//*************************** FUNKCIE PRE PRACU S REGISTRAMI ************************
//***********************************************************************************



//otvorenie kluca v registroch. 
//Navratova hodnota funkcie je handle vytvoreneho kluca.
//"lpKeyName" = meno a cesta podkluca, ktory sa ma odvorit al. vytvorit.
//Ak nastavime "lpbCreated" = TRUE, vrati TRUE, ak kluc neexistoval, a bol vytvoreny. Inak vrati NULL.
HKEY registry_OpenKey(LPCTSTR lpKeyName, LPBOOL lpbCreated)
{   DWORD dwDisp = 0;
    HKEY hkResult = NULL;
    if(RegCreateKeyEx(HKEY_CURRENT_USER, // handle existujuceho kluca v registroch (aktualny pouzivatel)
                      lpKeyName, //meno podkluca = vytvorenie vlastnej polozky v registroch, ktora bude stromovo zaradena pod "HKEY_CURRENT_USER".
                      0,  //tento parameter je rezervovany, a musi byt nula.
                      NULL,  //uzivatelom definovana trieda kluca
                      REG_OPTION_NON_VOLATILE, // nastavenia. REG_OPTION_NON_VOLATILE = ulozene hodnoty su zachovane aj po restarte PC. REG_OPTION_NON_VOLATILE = ulozene hodnoty su odstranene po restarte PC.
                      KEY_ALL_ACCESS, // maska, ktora urcuje pristupove prava pre kluce, ktore maju byt otvorene.
                      NULL, // ukazatel na strukturu "SECURITY_ATTRIBUTES" ktora urcuje, ci navratove handle moze byt dedene podriadenymi procesmi. NULL=handle nemozno dedit.
                      &hkResult, // ukazovatel na premennu, ktora obdrzi handle otvoreneho (existujuceho) al. vytvoreneho kluca.
                      &dwDisp) // ukazovatel na premennu, ktora dostane jednu z nasledujucichhodnot:...
                               // REG_CREATED_NEW_KEY = kluc neexistoval a bol vytvoreny
                               // REG_OPENED_EXISTING_KEY = kluc existoval, a bol len otvoreny
                               // NULL = niesu vratene ziadne informacie
       != ERROR_SUCCESS)   //navratova hodnota: ak funkcia uspeje, vrati ERROR_SUCCESS. V pripade chyby vrati kod chyby, ktore su definovane vo Winerror.h
       return NULL;
    //Ak na vstupe funkcie nastavim lpbCreated = TRUE, premenna vrati TRUE, ak kluc...
    //...neexistoval, a bol vytvoreny. V ostatnych pripadoch vrati NULL. (premenna je pointer)
    if (lpbCreated)
       *lpbCreated = (dwDisp & REG_CREATED_NEW_KEY);
    //funkcia vracia handle vytvoreneho kluca.   
    return hkResult;
}





//vytvorenie datovej hodnoty typu DWORD v kluci (polozke) registra.
BOOL registry_WriteBYTE(HKEY hKey, LPCTSTR lpValueName, BYTE bValue)
{   return (RegSetValueEx(hKey, //handle mnou vytvoreneho al. uz existujuceho kluca (spristupneneho na zapis).
                          lpValueName,  // nazov hodnoty, ktorym ju chcem nazvat. Ak je NULL alebo "", funkcia nastavi nazov na predvolenu hodnotu.
                          0,  //tento parameter je rezervovany, a musi byt nula.
                          REG_BINARY, // typ hodnoty. napr.: REG_DWORD (32bit) REG_BINARY, REG_NONE (bez definovania typu), REG_QWORD (64bit), atd...
                          (CONST BYTE*)&bValue, //pointer na premennu, v ktorej je hodnota, ktora ma byt ulozena do registra.
                          sizeof(BYTE)) // velkost ukladanej hodnoty (premennej) v bajtoch.
            == ERROR_SUCCESS);  //navratova hodnota: ak funkcia uspeje, vrati ERROR_SUCCESS. V pripade chyby vrati kod chyby, ktore su definovane vo Winerror.h
}






//vytvorenie datovej hodnoty typu DWORD v kluci (polozke) registra.
BOOL registry_WriteDWORD(HKEY hKey, LPCTSTR lpValueName, DWORD dwValue)
{   return (RegSetValueEx(hKey, //handle mnou vytvoreneho al. uz existujuceho kluca (spristupneneho na zapis).
                          lpValueName,  // nazov hodnoty, ktorym ju chcem nazvat. Ak je NULL alebo "", funkcia nastavi nazov na predvolenu hodnotu.
                          0,  //tento parameter je rezervovany, a musi byt nula.
                          REG_DWORD, // typ hodnoty. napr.: REG_DWORD (32bit) REG_BINARY, REG_NONE (bez definovania typu), REG_QWORD (64bit), atd...
                          (CONST BYTE*)&dwValue, //pointer na premennu, v ktorej je hodnota, ktora ma byt ulozena do registra.
                          sizeof(DWORD)) // velkost ukladanej hodnoty (premennej) v bajtoch.
            == ERROR_SUCCESS);  //navratova hodnota: ak funkcia uspeje, vrati ERROR_SUCCESS. V pripade chyby vrati kod chyby, ktore su definovane vo Winerror.h
}






//univerzalna funkcia na vytvorenie datovej hodnoty typu STRING v kluci (polozke) registra.
BOOL registry_WriteString(HKEY hKey, LPCTSTR lpValueName, LPCTSTR lpText)
{   return (RegSetValueEx(hKey, 
                          lpValueName, 
                          0, 
                          REG_SZ, // typ hodnoty. REG_SZ je typ "retazec zakonceny nulou"
                          (CONST BYTE*)lpText, //pointer na pole v ktorom su ulozene textove udaje, zakoncene nulou, ktore maju byt ulozene do registra.
                          (lstrlen(lpText)+1)*sizeof(TCHAR)) // velkost ukladanej hodnoty v bajtoch. ...lstrlen = dlzka textu * sizeof = dlzka datoveho typu textu v bajtoch. Cize celkovo dlzka textu v bajtoch. (1bajt = 8bitov)
            == ERROR_SUCCESS);
}






//citanie datovej hodnoty typu DWORD z kluca registra.
BOOL registry_ReadBYTE(HKEY hKey, LPCTSTR lpValueName, LPBYTE lpBResult)
{   BOOL bValue;
	DWORD bSize, bType;
    bSize = sizeof(BYTE);
    bValue = 0;
    if ( RegQueryValueEx(hKey, //handle mnou vytvoreneho al. uz existujuceho kluca (spristupneneho na zapis).
                         lpValueName, // nazov hodnoty, ktoru chcem nacitat. Ak nazov urcuje hodnotu ktora nieje v registri, funkcia vrati ERROR_FILE_NOT_FOUND. -Ak zadam NULL alebo "", funkcia nacita nemenovane al. predvolene hodnoty.
                         NULL, //tento parameter je rezervovany, a musi byt nula.
                         &bType, // typ hodnoty ktoru chcem nacitat. napr.: REG_DWORD (32bit) REG_BINARY, REG_NONE (bez definovania typu), REG_QWORD (64bit), atd...
                         (LPBYTE)&bValue, //ukazovatel na premennu/pole/vyrovnavaciu pamat, do ktorej sa ulozi hodnota al. data.
                         &bSize) //ukazovatel na premennu, v ktorej je zapisana velkost vyrovnavacej pamate/pola/premennej v bajtoch, kam sa uklada hodnota. -Ak pamat nieje dostatocne velka, funkcia vrati ERROR_MORE_DATA.
       != ERROR_SUCCESS )   //navratova hodnota: ak funkcia uspeje, vrati ERROR_SUCCESS. V pripade chyby vrati kod chyby, ktore su definovane vo Winerror.h
       return FALSE;
    *lpBResult = bValue;
    return TRUE;
}





//citanie datovej hodnoty typu DWORD z kluca registra.
BOOL registry_ReadDWORD(HKEY hKey, LPCTSTR lpValueName, LPDWORD lpdwResult)
{   DWORD dwValue, dwSize, dwType;
    dwSize = sizeof(DWORD);
    dwValue = 0;
    if ( RegQueryValueEx(hKey, //handle mnou vytvoreneho al. uz existujuceho kluca (spristupneneho na zapis).
                         lpValueName, // nazov hodnoty, ktoru chcem nacitat. Ak nazov urcuje hodnotu ktora nieje v registri, funkcia vrati ERROR_FILE_NOT_FOUND. -Ak zadam NULL alebo "", funkcia nacita nemenovane al. predvolene hodnoty.
                         NULL, //tento parameter je rezervovany, a musi byt nula.
                         &dwType, // typ hodnoty ktoru chcem nacitat. napr.: REG_DWORD (32bit) REG_BINARY, REG_NONE (bez definovania typu), REG_QWORD (64bit), atd...
                         (LPBYTE)&dwValue, //ukazovatel na premennu/pole/vyrovnavaciu pamat, do ktorej sa ulozi hodnota al. data.
                         &dwSize) //ukazovatel na premennu, v ktorej je zapisana velkost vyrovnavacej pamate/pola/premennej v bajtoch, kam sa uklada hodnota. -Ak pamat nieje dostatocne velka, funkcia vrati ERROR_MORE_DATA.
       != ERROR_SUCCESS )   //navratova hodnota: ak funkcia uspeje, vrati ERROR_SUCCESS. V pripade chyby vrati kod chyby, ktore su definovane vo Winerror.h
       return FALSE;
    *lpdwResult = dwValue;
    return TRUE;
}




//univerzalna funkcia na citanie datovej hodnoty typu STRING z kluca registra.
BOOL registry_ReadString(HKEY hKey, LPCTSTR lpValueName, LPTSTR lpString)
{   DWORD dwSize, dwType;
    //ulozi velkost celeho globalneho pola v bajtoch do dwSize
    dwSize = sizeof(lpString);
    if (RegQueryValueEx(hKey, //handle mnou vytvoreneho al. uz existujuceho kluca (spristupneneho na zapis).
                         lpValueName, // nazov hodnoty, ktoru chcem nacitat. Ak nazov urcuje hodnotu ktora nieje v registri, funkcia vrati ERROR_FILE_NOT_FOUND. -Ak zadam NULL alebo "", funkcia nacita nemenovane al. predvolene hodnoty.
                         NULL,  //tento parameter je rezervovany, a musi byt nula.
                         &dwType, // typ hodnoty ktoru chcem nacitat.
                         (LPBYTE)lpString, //ukazovatel na pole, do ktoreho sa ulozi text.
                         &dwSize) //ukazovatel na premennu, v ktorej je zapisana velkost textoveho pola v bajtoch, kam sa uklada text. -Ak pamat nieje dostatocne velka, funkcia vrati ERROR_MORE_DATA.
       != ERROR_SUCCESS )  //navratova hodnota: ak funkcia uspeje, vrati ERROR_SUCCESS. V pripade chyby vrati kod chyby, ktore su definovane vo Winerror.h
       return FALSE;
    return TRUE;
}




//zatvorenie kluca. 
BOOL registry_CloseKey(HKEY hKey)
{   return (RegCloseKey(hKey) == ERROR_SUCCESS);
}



//*********************************** ZAPIS/CITANIE REGISTROV ********************************



BOOL WriteRegistry(HWND hwnd)
{   
	RECT rectDSKTP;
	HWND hwndDSKTP = GetDesktopWindow();
	GetClientRect(hwndDSKTP, &rectDSKTP);
	
	WINDOWPLACEMENT wp;
    wp.length = sizeof(WINDOWPLACEMENT);
    //Naplni polozky struktury WINDOWPLACEMENT aktualnym stavom, rozmermi, atd, hl. okna.
    GetWindowPlacement(hwnd, &wp);
    //struktura obdlznika, ktora obsahuje polozky pre polohu laveho horneho...
    //...a praveho dolneho rohu obdlznika.
    RECT rect;
    //naplni strukturu rozmermi okna
    GetWindowRect(hwnd, &rect);
    // Osetrenie, ked je lavy horny roh okna za rohom obrazovky. 
    //Vdaka tomu sa do registra sa ulozi hodnota minimalne 0.
    if(rect.left < 0)
    {  rect.right -= rect.left;
       rect.left = 0;             
    }
    if(rect.top < 0)
    {  rect.bottom -= rect.top;
       rect.top = 0;
    }
    if(rect.bottom > rectDSKTP.bottom)
    {	rect.top -= (rect.bottom - rectDSKTP.bottom);
		rect.bottom = rectDSKTP.bottom;
    }
    if(rect.right > rectDSKTP.right)
    {	rect.left -= (rect.right - rectDSKTP.right);
    	rect.right = rectDSKTP.right;
    }
    // Zo struktury WINDOWPLACEMENT sa do premennej registra ulozi udaj o stave okna pred...
    //...jeho zatvorenim. (tj. ci bolo maximalizovane, normalne alebo minimalizovane)
    g_dwWindowShow = wp.showCmd;
    //Udaje o rozmeroch okna sa do registra zapisu len vtedy, ked nebolo pred zatvorenim...
    //...maximalizovane al. minimalizovane. (tj. bolo zobrazene normalne) 
    if(wp.showCmd == SW_SHOWNORMAL)
    {   //MAKELONG zluci suradnice x,y laveho horneho rohu okna do jedinej 32bitovej globalnej...
        //...premennej, ako horny a dolny WORD. Na ich rozdelenie potom treba pouzit LOWORD(),HIWORD().
        g_dwWindowPos = MAKELONG(rect.left, rect.top);
        //MAKELONG zluci sirku a vysku okna do jedinej 32bitovej globalnej premennej, ako horny...
        //...a dolny WORD. Na ich rozdelenie potom treba pouzit LOWORD(),HIWORD().
        g_dwWindowSize = MAKELONG(rect.right-rect.left, rect.bottom-rect.top);
    }
    //Zapis do registrov
	BOOL bRes = TRUE;
    HKEY hKey = registry_OpenKey(REGISTRY_KEY_NAME, NULL);
    if ( !hKey )
       return FALSE;
    if (!registry_WriteDWORD(hKey, TEXT("WindowPos"), g_dwWindowPos))
       bRes = FALSE;
    if (!registry_WriteDWORD(hKey, TEXT("WindowSize"), g_dwWindowSize))
       bRes = FALSE;
    if (!registry_WriteDWORD(hKey, TEXT("WindowShow"), g_dwWindowShow))
       bRes = FALSE;
     if (!registry_CloseKey(hKey))
       bRes = FALSE;
    return bRes;
}






BOOL ReadRegistry()
{   
	RECT rectDSKTP;
	HWND hwndDSKTP = GetDesktopWindow();
	GetClientRect(hwndDSKTP, &rectDSKTP);
	// Umiestnenie okna pri prvom spusteni do stredu prac. plochy
	int x_okna_first = (rectDSKTP.right-rectDSKTP.left)/2-_SirkaHlOkna/2,
		y_okna_first = (rectDSKTP.bottom-rectDSKTP.top)/2-_VyskaHlOkna/2;
	
	BOOL bRes = TRUE;
    BOOL bFirst = FALSE;
    HKEY hKey = registry_OpenKey(REGISTRY_KEY_NAME, &bFirst);
    if (!hKey)
        return FALSE;
    if (bFirst)
    {   g_dwWindowPos = MAKELONG(x_okna_first, y_okna_first); //x,y. Zlucene do dolneho a horneho wordu.
        g_dwWindowSize = MAKELONG(_SirkaHlOkna, _VyskaHlOkna); //vyska, sirka. Zlucene do dolneho a horneho wordu.
        g_dwWindowShow = 1;
        return TRUE;
    }
    if ( !registry_ReadDWORD(hKey, "WindowPos", &g_dwWindowPos) )
        bRes = FALSE;
    if ( !registry_ReadDWORD(hKey, "WindowSize", &g_dwWindowSize) )
        bRes = FALSE;
    if ( !registry_ReadDWORD(hKey, "WindowShow", &g_dwWindowShow) )
        bRes = FALSE;     
   	if ( !registry_ReadBYTE(hKey, "Baud/s", &Aktualna_rychlost) )
        bRes = FALSE;     
   	if ( !registry_ReadBYTE(hKey, "Port", &posledny_pripojeny_port) )
        bRes = FALSE;     
    if ( !registry_CloseKey(hKey) )
        bRes = FALSE;
    return bRes;
}











//*************************************************************
//*************************** PAUZY  **************************
//*************************************************************



void Wait_ms(int prodleva)
{
 int stop ;
  timeBeginPeriod (1);
  stop = timeGetTime() + prodleva;
  while (stop > timeGetTime());
  timeEndPeriod (1);
}





//**********************************************************************
//*************************** FUNKCIE PISANIA **************************
//**********************************************************************


void Text(	HDC hdc, 
			SIZE* rozmery, //vrati rozmery textu
			LPCTSTR text,  //text, alebo text s formatovacim znakom, napr. %d v pripade, ze chcem zobrazit aj premennu.
			int premenna,  //premenna na zobrazenie.
			int x, int y,  // umiestnenie textu
			COLORREF farba_pisma, // farba pisma
			int velkost_pisma, // velkost pisma. Najlepsie je zadva zporn hodnoty. Velkost sa vtedy zobrazi presne.
			int pismo_bold, //  1 tun psmo, 0 normlne psmo.
			LPCTSTR typ_pisma ) //Nzov typu psma vo forme textu v vodzovkch
{	LOGFONT logFont;
	TCHAR textTMP[1000];
	int GraphicModeOld = SetGraphicsMode (hdc, GM_ADVANCED);	//Graficky rezim podporujuci zmenu logFont.lfOrientation pisma
	//GraphicModeOld = SetGraphicsMode (hdc, GM_COMPATIBLE);
	SetBkMode(hdc, TRANSPARENT);
	COLORREF ColorOld = SetTextColor(hdc, farba_pisma);
	GetObject(GetStockObject(DEFAULT_GUI_FONT), sizeof(logFont), &logFont); 
	//lstrcpy(logFont.lfFaceName, "Segoe UI");
	lstrcpy(logFont.lfFaceName, typ_pisma);
	//Velkost pisma
	logFont.lfHeight = -MulDiv (velkost_pisma, GetDeviceCaps (hdc, LOGPIXELSY), 72);
	//logFont.lfWidth = 13; //Sirka pisma (ak je 0, pouzije sa automaticky najblizsia hodnota)
	if(pismo_bold)
		logFont.lfWeight = 600; //Tucne pismo
	else
		logFont.lfWeight = 0;
	//logFont.lfEscapement = 0;//-300;//900; //Uhol celeho textu (rotacia)
	//logFont.lfItalic = TRUE;
	//logFont.lfOrientation = 0;//-300;//900; //Zvisly text		
	HFONT hFont = CreateFontIndirect(&logFont);
	HFONT hFontOld = (HFONT)SelectObject(hdc, hFont); 
	//---------------------------------------------------------------------------------				    
	_stprintf(textTMP, text, premenna);		
	TextOut(hdc, x, y, textTMP, lstrlen(textTMP));
	GetTextExtentPoint32(hdc, textTMP, lstrlen(textTMP), rozmery);							
	//---------------------------------------------------------------------------------				    
	SelectObject(hdc, hFontOld);
	DeleteObject(hFont);
	SetTextColor(hdc, ColorOld);
	SetGraphicsMode (hdc, GraphicModeOld);
}







RECT RozmeryOkna(HWND hwnd)
{
	//Ziska aktualnu polohu laveho horneho okraja okna
	RECT rectDSKTP;
	HWND hwndDSKTP = GetDesktopWindow();
	GetClientRect(hwndDSKTP, &rectDSKTP);
	WINDOWPLACEMENT wp;
    wp.length = sizeof(WINDOWPLACEMENT);
    GetWindowPlacement(hwnd, &wp);
    RECT rect;
    GetWindowRect(hwnd, &rect);
    return rect;
}





void Text_stred(HWND hwnd,	
				HDC hdc, 
				SIZE* rozmery, //vrati rozmery textu
				LPCTSTR text,  //text, alebo text s formatovacim znakom, napr. %d v pripade, ze chcem zobrazit aj premennu.
				int premenna,  //premenna na zobrazenie.
				int y,  // umiestnenie textu
				COLORREF farba_pisma, // farba pisma
				int velkost_pisma, // velkost pisma. Najlepsie je zadva zporn hodnoty. Velkost sa vtedy zobrazi presne.
				int pismo_bold, //  1 tun psmo, 0 normlne psmo.
				LPCTSTR typ_pisma ) //Nzov typu psma vo forme textu v vodzovkch
	{	LOGFONT logFont;
	TCHAR textTMP[1000];
	int GraphicModeOld = SetGraphicsMode (hdc, GM_ADVANCED);	//Graficky rezim podporujuci zmenu logFont.lfOrientation pisma
	//GraphicModeOld = SetGraphicsMode (hdc, GM_COMPATIBLE);
	SetBkMode(hdc, TRANSPARENT);
	COLORREF ColorOld = SetTextColor(hdc, farba_pisma);
	GetObject(GetStockObject(DEFAULT_GUI_FONT), sizeof(logFont), &logFont); 
	//lstrcpy(logFont.lfFaceName, "Segoe UI");
	lstrcpy(logFont.lfFaceName, typ_pisma);
	//Velkost pisma
	logFont.lfHeight = -MulDiv (velkost_pisma, GetDeviceCaps (hdc, LOGPIXELSY), 72);
	//logFont.lfWidth = 13; //Sirka pisma (ak je 0, pouzije sa automaticky najblizsia hodnota)
	if(pismo_bold)
		logFont.lfWeight = 600; //Tucne pismo
	else
		logFont.lfWeight = 0;
	//logFont.lfEscapement = 0;//-300;//900; //Uhol celeho textu (rotacia)
	//logFont.lfItalic = TRUE;
	//logFont.lfOrientation = 0;//-300;//900; //Zvisly text		
	HFONT hFont = CreateFontIndirect(&logFont);
	HFONT hFontOld = (HFONT)SelectObject(hdc, hFont); 
	//----------------------------------------------------------------------------------
	RECT rect = RozmeryOkna(hwnd);			    
	_stprintf(textTMP, text, premenna);			
	GetTextExtentPoint32(hdc, textTMP, lstrlen(textTMP), rozmery);	
	TextOut(hdc, (rect.right-rect.left-rozmery->cx/1)/2, y, textTMP, lstrlen(textTMP));					
	//---------------------------------------------------------------------------------				    
	SelectObject(hdc, hFontOld);
	DeleteObject(hFont);
	SetTextColor(hdc, ColorOld);
	SetGraphicsMode (hdc, GraphicModeOld);
}








void Text_double(	HDC hdc, 
				SIZE* rozmery, //vrati rozmery textu
				LPCTSTR text,  //text, alebo text s formatovacim znakom, napr. %d v pripade, ze chcem zobrazit aj premennu.
				double premenna,  //premenna na zobrazenie.
				int x, int y,  // umiestnenie textu
				COLORREF farba_pisma, // farba pisma
				int velkost_pisma, // velkost pisma. Najlepsie je zadva zporn hodnoty. Velkost sa vtedy zobrazi presne.
				int pismo_bold, //  1 tun psmo, 0 normlne psmo.
				LPCTSTR typ_pisma ) //Nzov typu psma vo forme textu v vodzovkch
	{	LOGFONT logFont;
	TCHAR textTMP[1000];
	int GraphicModeOld = SetGraphicsMode (hdc, GM_ADVANCED);	//Graficky rezim podporujuci zmenu logFont.lfOrientation pisma
	//GraphicModeOld = SetGraphicsMode (hdc, GM_COMPATIBLE);
	SetBkMode(hdc, TRANSPARENT);
	COLORREF ColorOld = SetTextColor(hdc, farba_pisma);
	GetObject(GetStockObject(DEFAULT_GUI_FONT), sizeof(logFont), &logFont); 
	//lstrcpy(logFont.lfFaceName, "Segoe UI");
	lstrcpy(logFont.lfFaceName, typ_pisma);
	//Velkost pisma
	logFont.lfHeight = -MulDiv (velkost_pisma, GetDeviceCaps (hdc, LOGPIXELSY), 72);
	//logFont.lfWidth = 13; //Sirka pisma (ak je 0, pouzije sa automaticky najblizsia hodnota)
	if(pismo_bold)
		logFont.lfWeight = 600; //Tucne pismo
	else
		logFont.lfWeight = 0;
	//logFont.lfEscapement = 0;//-300;//900; //Uhol celeho textu (rotacia)
	//logFont.lfItalic = TRUE;
	//logFont.lfOrientation = 0;//-300;//900; //Zvisly text		
	HFONT hFont = CreateFontIndirect(&logFont);
	HFONT hFontOld = (HFONT)SelectObject(hdc, hFont); 
	//---------------------------------------------------------------------------------				    
	_stprintf(textTMP, text, premenna);		
	TextOut(hdc, x, y, textTMP, lstrlen(textTMP));
	GetTextExtentPoint32(hdc, textTMP, lstrlen(textTMP), rozmery);							
	//---------------------------------------------------------------------------------				    
	SelectObject(hdc, hFontOld);
	DeleteObject(hFont);
	SetTextColor(hdc, ColorOld);
	SetGraphicsMode (hdc, GraphicModeOld);
}



//***********************************************************************************************
//*************************** FUNKCIE PRE VYKRESLENIE GRAFICKYCH TVAROV **************************
//***********************************************************************************************

//Obdlznik so zaoblenymi hranami
void ObdlznikZaobleny(HDC hdc, int x, int y, int sirka, int vyska, int zaoblenie, int hrubka_pera, COLORREF farba_pera, COLORREF farba_stetca)
{	HPEN hPen = CreatePen(PS_SOLID | PS_INSIDEFRAME, hrubka_pera, farba_pera);  //0x00FF0000);   
  	HPEN hPenOld = (HPEN)SelectObject(hdc, hPen);  
  	HBRUSH hBrush = CreateSolidBrush(farba_stetca);  //0x0080FFFF); 
 	HBRUSH hBrushOld = (HBRUSH)SelectObject(hdc, hBrush);
  	RoundRect(hdc, x, y, x + sirka, y + vyska, zaoblenie, zaoblenie);	
  	SelectObject(hdc, hPenOld);
  	SelectObject(hdc, hBrushOld);
  	DeleteObject(hPen); 
    DeleteObject(hBrush);
}


//Obdlznik
void Obdlznik(HDC hdc, int x, int y, int sirka, int vyska, int hrubka_pera, COLORREF farba_pera, COLORREF farba_stetca)
{	HPEN hPen = CreatePen(PS_SOLID | PS_INSIDEFRAME, hrubka_pera, farba_pera);
  	HPEN hPenOld = (HPEN)SelectObject(hdc, hPen);  
  	HBRUSH hBrush = CreateSolidBrush(farba_stetca);  
 	HBRUSH hBrushOld = (HBRUSH)SelectObject(hdc, hBrush);
  	Rectangle(hdc, x, y, x + sirka, y + vyska);	
  	SelectObject(hdc, hPenOld);
  	SelectObject(hdc, hBrushOld);
  	DeleteObject(hPen); 
    DeleteObject(hBrush);
}


// Obluk lubovolnych rozmerov
void Obluk(HDC hdc, int x, int y, int polomer, int uhol_zaciatku, int uhol_posunu, int hrubka_pera, COLORREF farba_pera)
{	HPEN hPen = CreatePen(PS_SOLID, hrubka_pera, farba_pera);
    HPEN hpOld = (HPEN)SelectObject(hdc, hPen);
    MoveToEx(hdc, x + polomer*cos((uhol_zaciatku*_PI)/180), y - polomer*sin((uhol_zaciatku*_PI)/180), NULL); //bod pociatku sleduje zaciatok obluka
    AngleArc(hdc, x, y, polomer, uhol_zaciatku, uhol_posunu);   
    SelectObject(hdc, hpOld);  
    DeleteObject(hPen); 
}


void Usecka(HDC hdc, int x, int y, int dlzka, int vyska, int hrubka_pera, COLORREF farba_pera)
{   HPEN hPen= CreatePen(PS_SOLID, hrubka_pera, farba_pera);
    HPEN hPenOld = (HPEN)SelectObject(hdc, hPen);
    MoveToEx(hdc, x, y, NULL);
    LineTo(hdc, x+dlzka, y-vyska);
    SelectObject(hdc, hPenOld);
    DeleteObject(hPen);
}


//Elipsa
void Elipsa(HDC hdc, int x, int y, int sirka, int vyska, int hrubka_pera, COLORREF farba_pera, COLORREF farba_stetca)
{	HPEN hPen = CreatePen(PS_SOLID | PS_INSIDEFRAME, hrubka_pera, farba_pera);
  	HPEN hPenOld = (HPEN)SelectObject(hdc, hPen);  
  	HBRUSH hBrush = CreateSolidBrush(farba_stetca);  
 	HBRUSH hBrushOld = (HBRUSH)SelectObject(hdc, hBrush);
  	Ellipse(hdc, x, y, x + sirka, y + vyska);
  	SelectObject(hdc, hPenOld);
  	SelectObject(hdc, hBrushOld);
  	DeleteObject(hPen); 
    DeleteObject(hBrush);
}


//Vlastny tvar 
void VlastnyTvar(HDC hdc, int x, int y, int sirka, int vyska, int hrubka_pera, COLORREF farba_pera)
{   //definovanie bodov nadvazujucich useciek
    POINT MyPoints[5] = {x,y, x,y+vyska, x+sirka,y, x+sirka,y+vyska, x,y};
    HPEN hPen = CreatePen(PS_SOLID, hrubka_pera, farba_pera);
    HPEN hpOld = (HPEN)SelectObject(hdc, hPen);  
    Polyline(hdc, MyPoints, 5);
    SelectObject(hdc, hpOld);
    DeleteObject(hPen);
}


//Elipsa
void VlastnyTvarVyplneny(HDC hdc, int x, int y, int sirka, int vyska, int hrubka_pera, COLORREF farba_pera, COLORREF farba_stetca)
{	//definovanie bodov nadvazujucich useciek
    POINT MyPoints[5] = {x,y, x,y+vyska, x+sirka,y, x+sirka,y+vyska, x,y};
	HPEN hPen = CreatePen(PS_SOLID | PS_INSIDEFRAME, hrubka_pera, farba_pera);
  	HPEN hPenOld = (HPEN)SelectObject(hdc, hPen);  
  	HBRUSH hBrush = CreateSolidBrush(farba_stetca);  
 	HBRUSH hBrushOld = (HBRUSH)SelectObject(hdc, hBrush);
  	Polygon(hdc, MyPoints, 5);
  	SelectObject(hdc, hPenOld);
  	SelectObject(hdc, hBrushOld);
  	DeleteObject(hPen); 
    DeleteObject(hBrush);
}




void LedSvetlo(HDC hdc, int farba_pera, int farba_stetca, int zaoblenie, RECT rect)
{	HPEN hPen, hPenOld;
  	HBRUSH hBrush, hBrushOld;
  	hPen = CreatePen(PS_SOLID | PS_INSIDEFRAME, 1, farba_pera);  //0x00FF0000);   
  	hPenOld = (HPEN)SelectObject(hdc, hPen);  
  	hBrush = CreateSolidBrush(farba_stetca);  //0x0080FFFF); 
 	hBrushOld = (HBRUSH)SelectObject(hdc, hBrush);
  	RoundRect(hdc, rect.left, rect.top, rect.right, rect.bottom, zaoblenie, zaoblenie);	
  	SelectObject(hdc, hPenOld);
  	SelectObject(hdc, hBrushOld);
  	DeleteObject(hPen); 
    DeleteObject(hBrush);
	
}





/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////// RS232 FUNKCIE ////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////



// Vyhlada cisla aktivnych COM portov.
int SearchCom(int cislo_portu)
{	char MenoComu[6];	
	CloseHandle(ComHandle);	
	sprintf(MenoComu,"COM%d",cislo_portu);	
	ComHandle = CreateFile(MenoComu,
						   GENERIC_READ|GENERIC_WRITE,
			               0,
						   0,
						   OPEN_EXISTING,
						   FILE_FLAG_OVERLAPPED,
						   0);						
	CloseHandle(ComHandle);		
	if(ComHandle==INVALID_HANDLE_VALUE) 
		return 0;
	else 
		return 1;	
}





// Otvori COM port
bool OpenCom(HWND hwnd, int CisloComu)
{	char MenoComu[6];
	int bit, parity;
	CloseHandle(ComHandle);	
	sprintf(MenoComu, "COM%d", CisloComu);
	ComHandle = CreateFile(MenoComu,
						   GENERIC_READ|GENERIC_WRITE,
			               FILE_SHARE_READ | FILE_SHARE_WRITE,//NULL
						   0,
						   OPEN_EXISTING,
						   FILE_FLAG_OVERLAPPED,
						   0);						
	if(ComHandle==INVALID_HANDLE_VALUE) 
	{	MessageBox(hwnd, "COM port sa ned otvori.", "Chyba", MB_OK | MB_ICONINFORMATION);
		return (0);
	}
	if(!SetCommMask(ComHandle, EV_RXCHAR | EV_TXEMPTY))
	{	MessageBox(hwnd, "Ned sa nastavi maska.", "Chyba", MB_OK | MB_ICONSTOP);
		return (0);
	}		
	osRead.hEvent = CreateEvent(NULL,TRUE,FALSE,NULL);
	osWrite.hEvent = CreateEvent(NULL,TRUE,FALSE,NULL);	
	if((osRead.hEvent==NULL) || (osWrite.hEvent==NULL))
 	{	MessageBox(hwnd, "Nedaj sa vytvori objekty udalost.", "Chyba", MB_OK | MB_ICONSTOP);
		return (0);
	}	
	memset(&osRead, 0, sizeof(osRead));	
	memset(&osWrite, 0, sizeof(osWrite));	
	if(!SetupComm(ComHandle, 4096, 4096))
	{	MessageBox(hwnd, "Ned sa nastavi vekos bufferov.", "Chyba", MB_OK | MB_ICONSTOP);
		return (0);
	}
	if(!PurgeComm(ComHandle, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR))
	{	MessageBox(hwnd, "Nedaj sa vyprzdni buffery.", "Chyba", MB_OK | MB_ICONSTOP);
		return (0);
	}
	dcb.BaudRate = baudova_rychlost; // Rychlost prenosu RS232
	dcb.ByteSize = 8; 
	dcb.DCBlength = sizeof(dcb);
	dcb.EofChar = 0;
	dcb.ErrorChar = 0;
	dcb.EvtChar = 0;
	dcb.fAbortOnError = FALSE;
	dcb.fBinary = TRUE;	
	dcb.fDsrSensitivity = FALSE;					//DSR citlivost
	dcb.fDtrControl = DTR_CONTROL_ENABLE; 			//DTR_CONTROL_DISABLE;//1;		//DTR obvod
	dcb.fDummy2 = 0;
	dcb.fErrorChar = FALSE;
	dcb.fInX = FALSE;								//XON / XOF
	dcb.fNull = FALSE;
	dcb.fOutX = FALSE;								//XON / XOF
	dcb.fOutxCtsFlow = FALSE;//TRUE;				//CTS synchronizacia dohovorom	
	dcb.fOutxDsrFlow = FALSE;						//DSR synchronizacia dohovorom
	dcb.fParity = FALSE;
	dcb.fRtsControl =RTS_CONTROL_HANDSHAKE;			// 0;//RTS_CONTROL_ENABLE; //RTS_CONTROL_HANDSHAKE;		//RTS obvod
	dcb.fTXContinueOnXoff = FALSE;
	dcb.Parity = NOPARITY;
	dcb.StopBits = ONESTOPBIT;
	dcb.wReserved = 0;
	dcb.wReserved1 = 0;
	dcb.XoffChar = 0;
	dcb.XoffLim = 0;
	dcb.XonChar = 0;
	dcb.XonLim = 0;			
	if(!SetCommState(ComHandle, &dcb))
	{	MessageBox(hwnd, "Ned sa nastavi COM poda DCB truktry.", "Chyba", MB_OK | MB_ICONSTOP);
		return (0);
	}
	return (1);
}






// Zatvorenie portu a udalostii, vynulovanie masky, vycistenie bufferov.
void CloseCom()
{	PurgeComm(ComHandle,PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);
	SetCommMask(ComHandle,0);
	CloseHandle(ComHandle);
	ComHandle = NULL;
	CloseHandle(osRead.hEvent);
	CloseHandle(osWrite.hEvent);
	SuspendThread(ghThread);
}










Hladanie_com_portov(HWND hwnd)
{	int f,n;
	char MenoComu[6];
 	//Vynuluje pole pre zapis aktivnych COM portov
	for(f = 0; f < 100; f++)
		AktivnePorty[f] = 0;
	n = 0;
	//Zapise do pola aktivne COM porty
	for(f = 0; f < 100; f++)
	{	if(SearchCom(f))
		{	AktivnePorty[n++] = f;
			existuje_port = n;
		}
	}
	n--;
	for(f = 0; f < 100; f++)
	{	if((AktivnePorty[f] < AktivnePorty_old[f]) && (AktivnePorty[f] != 0))
		{	posledny_pripojeny_port = f;			
			break;
		}
		else
			posledny_pripojeny_port = n;
	}
	//urobi inicializaciu iba vtedy, ked sa port zmenil, alebo zacal existovat (zisti to porovnanim s predoslym stavom v pamati)
	if(porty_pamat != existuje_port)
	{	for(f = 0; f <= porty_pamat; f++)	
			SendDlgItemMessage(hwnd, ID_COMBO2, CB_DELETESTRING, 0, 0);
		for(f = 0; f <= n; f++)
		{	sprintf(MenoComu, "COM %d", AktivnePorty[f]);
			SendDlgItemMessage(hwnd, ID_COMBO2, CB_ADDSTRING, 0, (LPARAM)MenoComu);
		}
		//Skopiruje aktivne porty po zmene do pola historie aktivnych portov
		for(f = 0; f < 100; f++)	
			AktivnePorty_old[f] = AktivnePorty[f];
		SendDlgItemMessage(hwnd, ID_COMBO2, CB_SETCURSEL, posledny_pripojeny_port, 0);		
		//Zapis posledneho vlozeneho portu do registrov
 		HKEY hKey;
 		if((hKey = registry_OpenKey(REGISTRY_KEY_NAME, NULL)))
			registry_WriteBYTE(hKey, TEXT("Port"), posledny_pripojeny_port);
		registry_CloseKey(hKey);
		//Prekreslenie okna
		RedrawWindow(hwnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE);
		//ulozi do pamati informaciu o existencii portu pre buduce porovnanie
		porty_pamat = existuje_port; 
	}		
}












Init_com_portov(HWND hwnd)
{	int f,n;
	char MenoComu[6];
 	//Vynuluje pole pre zapis aktivnych COM portov
	for(f = 0; f < 100; f++)
		AktivnePorty[f] = 0;
	n = 0;
	//Zapise do pola aktivne COM porty
	for(f = 0; f < 100; f++)
	{	if(SearchCom(f))
		{	AktivnePorty[n] = f;
			AktivnePorty_old[n++] = f;
			existuje_port = n;
		}
	}
	n--;
	
	if(n > -1)
	{
		for(f = 0; f <= porty_pamat; f++)	
			SendDlgItemMessage(hwnd, ID_COMBO2, CB_DELETESTRING, 0, 0);
		for(f = 0; f <= n; f++)
		{	sprintf(MenoComu, "COM %d", AktivnePorty[f]);
			SendDlgItemMessage(hwnd, ID_COMBO2, CB_ADDSTRING, 0, (LPARAM)MenoComu);
		}
		//Skopiruje aktivne porty po zmene do pola historie aktivnych portov
		for(f = 0; f < 100; f++)	
			AktivnePorty_old[f] = AktivnePorty[f];		
		if(n < posledny_pripojeny_port)
			posledny_pripojeny_port = n;
		SendDlgItemMessage(hwnd, ID_COMBO2, CB_SETCURSEL, posledny_pripojeny_port, 0);			
		RedrawWindow(hwnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE);
		//ulozi do pamati informaciu o existencii portu pre buduce porovnanie
		porty_pamat = existuje_port; 
	}
}






